// Returns the number of results for a particular search query
// loadStrings() didn't cut it, Google returns 403s unless you fib the header a bit.
// Runs in its own thread so we don't clutter up the draw loop or halt renderiung.

// Could have used the Yahoo API with much less grief, but Google was a
// better conceptual fit.

import processing.net.*;

class GoogleScraper implements Runnable {
  Client client;
  String html;
  PApplet parent;
  Thread scraperThread;
  String latestWord;
  String query;
  Long resultCount;
  int startTime;
  int timeOut;
  boolean runThread;
  boolean timedOut;
  boolean noResults;
  boolean fetching;
  boolean success;
  

  GoogleScraper(PApplet _parent) {
    parent = _parent;
    client = new Client(parent, "www.google.com", 80);

    // Create the thread supplying it with the runnable object.
    scraperThread = new Thread(this);

    startTime = 0;
    timeOut = 5000; // Give up after five seconds
    timedOut = false;
    noResults = false;
    fetching = false;
    success = false;

    // Start the thread.
    runThread = true;
    scraperThread.start();
    
  }

  // One time use...
  void getCount(String _query) {
    query = _query;
    fetching = true;

    println("Searching for \"" + query + "\" with query \"" + URLEncoder.encode(query) + "\".");
    
    // Ham up the header a bit so we don't get 403'd.
    // Encode the URL for more advanced queries. 
    client.write("GET /search?q=" + URLEncoder.encode(query) + " HTTP/1.1\n");
    client.write("Host: www.google.com\n\n");
    client.write("User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)\n\n");

    startTime = millis();
  }


  void gotResult() {
      fetching = false;
      runThread = false;
    
      // This is kludgey... the result count is always the third bolded number.
      if(!html.equals("")) {
        String[][] matches = matchAll(html, "<b>[0-9,]+</b>");
      
        println(matches);
      
        if((matches != null) && (matches.length >= 3)) {
          String rawResult = matches[matches.length - 1][0];
          resultCount = Long.parseLong(rawResult.substring(3, rawResult.length() - 4).replaceAll(",", ""));
          println(resultCount);
          success = true;
        }
        else {
          println("No results.");
          noResults = true;
        }
      }
      else {
        println("Timed out.");
        timedOut = true;
      }
  }

  void run() {
    // Always running.
    while(true) {
      
      if(runThread) {
      
        if ((latestWord != query) && (client.available() > 0)) {
          html += client.readString();

          // Stop looking once we're past the point of interest.
          if(html.indexOf("<h2 class=hd>") > 0) {
            latestWord = query;
            //client.stop();
            //client.clear();
            //println(html);
            gotResult();
          }
        }
      
        // Give up if it's taking too long.
        if ((startTime > 0) && ((millis() - startTime) > timeOut)) {
          latestWord = "";
          html = "";
          startTime = 0;
          gotResult();

        }
      
      }

      try {
        // Don't hog the CPU.
        scraperThread.sleep(20);
      }
      catch(InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
  
}
